$ifndef _KS0713_
$define _KS0713_
'
' Macros to interface to the Samsung KS0713 128x64 graphic LCD chipset
'
' The routines make use of SFRs as temporary variables as much as possible
' in order to minimise the imact of variable usage on the main program.
'
' Written by Les Johnson - 31st May 2010
' For use with the Crownhill Amicus18 BASIC compiler version 1.0.0.8 onwards
'
' This source code is supplied as-is, and no guarantees are offered on it's operation
'
Warnings = Off
Reminders = Off
    #disable Print                          ' Remove the compiler's Print library subroutine
'
' Default user defines
'
'    $define GLCD_Flip                      ' Uncomment to reverse the display's positioning
'    $define GLCD_Negative                  ' Uncomment for a negative LCD display
'
$ifndef GLCD_CS1
    $define GLCD_CS1 PORTC.2                ' LCD's CS1 pin
$endif
$ifndef GLCD_Reset
    $define GLCD_Reset PORTC.4              ' LCD's Reset pin
$endif
$ifndef GLCD_A0
    $define GLCD_A0 PORTC.1                 ' LCD's A0 pin, sometimes named the A0 pin
$endif
$ifndef GLCD_RW
    $define GLCD_RW PORTA.5                 ' LCD's RW
$endif
$ifndef GLCD_EN_RD
    $define GLCD_EN_RD PORTA.4              ' LCD's En or RD pin
$endif
$ifndef GLCD_Data
    $define GLCD_Data PORTB                 ' LCD's data port
$endif

$ifndef cGlcd_Width
    $define cGlcd_Width 128                 ' The Width of the LCD (in pixels)
$endif
$ifndef cGlcd_Height
    $define cGlcd_Height 64                 ' The hight of the LCD (in pixels)
$endif
$ifndef cGlcd_InitDelay
    $define cGlcd_InitDelay 100             ' LCD initialise delay (in milliseconds)
$endif
'
' Create some compiler system(scratch) variables
'
    Dim PP0 As Byte System                  ' Data byte to the LCD parameter
    Dim PP0H As Byte System                 ' Command byte to the LCD parameter
    Dim PP1 As Byte System                  ' Used as a timeout when waiting for the LCD's response
    Dim PP2 As Byte System
    Dim PP2H As Byte System
    Dim PP3 As Byte System
    Dim PP3H As Byte System
    Dim PP6 As Byte System
    Dim PP6H As Byte System
    Dim GEN2 As Byte System                 ' Xpos Start parameter
    Dim GEN2H As Byte System                ' Ypos Start parameter
'
' Variables used in the routines
'
    Dim bGlcd_Data As PP0
    Dim bhCommand As PP0H
    Dim bGlcd_WaitTimeout As PP1            ' Used as a timeout when waiting for the LCD's response
    Dim bGlcd_Xpos As Byte System
    Dim bGlcd_Ypos As Byte System
    Dim bColour As Byte System              ' Holds the colour of the pixel (0 = off, 1 = On, 2 = Xor)
'
' Glcd_Circle variables
'
    Dim bGlcd_QuadA As PP2
    Dim bGlcd_QuadB As PP2H
    Dim bGlcd_QuadC As PP6
    Dim bGlcd_QuadD As PP6H

    Dim bGlcd_Xpos_S As GEN2
    Dim bGlcd_Ypos_S As GEN2H

    Dim bGlcd_Size As PP3                       ' Size of a circle, or a line, or a box
    Dim bGlcd_Height As PP3                     ' Height of the rectangle
    Dim bGlcd_Length As PP3H                    ' Length of the rectangle

    Dim bGlcd_TR As Byte System
    Dim bGlcd_DD As Byte System

    Dim bGlcd_XX As Byte System
    Dim bGlcd_YY As Byte System
'
' Glcd_Line variable aliased from circle variables
'
    Dim bGlcd_Xpos_E As bGlcd_QuadA
    Dim bGlcd_Ypos_E As bGlcd_QuadB
    Dim bGlcd_DeltaY As bGlcd_QuadC
    Dim bGlcd_DeltaX As bGlcd_QuadD

    Dim bGlcd_StepY As bGlcd_XX
    Dim bGlcd_Deviation As bGlcd_TR
    Dim bGlcd_StepX As bGlcd_DD
'
' Glcd_Plot variable aliases
'
    Dim bGlcd_PixelX As TBLPTRL              ' Temporary pixel Xpos holder
    Dim bGlcd_PixelY As TBLPTRH              ' Temporary pixel Ypos holder
    Dim bGlcd_gPixel As EEDATA               ' Temporary storage for pixel condition
'
' Glcd_PrintChar variable aliases
'
    Dim bGlcd_CharSave As FSR1L              ' Temporary storage for the character to display
    Dim wGlcd_CharOffset As PRODL.Word       ' Font offset within it's code memory table
    Dim bGlcd_LcdLoop As FSR1H               ' Temporary loop
    Dim wGlcd_TblPtrSave As Word System      ' Saves the value of register pair TBLPTRL\H
'
' Glcd_Print variables
'
    Dim wGlcd_FSR0 As FSR0L.Word
    Dim wGlcd_FSR1 As FSR1L.Word
    Dim wGlcd_Tblptr As TBLPTRL.Word
'
' KS0713 LCD control commands
'
    $define cCmd_On $AF
    $define cCmd_Off $AE
    $define cCmd_StartLine $40
    $define cCmd_Page $B0
    $define cCmd_ColumnMS $10
    $define cCmd_ColumnLS $0F
    $define cCmd_ADCNormal $A0
    $define cCmd_ADCReverse $A1
    $define cCmd_DisplayNormal $A6
    $define cCmd_DisplayReverse $A7
    $define cCmd_DisplayAllOff $A4
    $define cCmd_DisplayAllOn $A5
    $define cCmd_Bias19 $A2
    $define cCmd_Bias17 $A3
    $define cCmd_Modify $E0
    $define cCmd_End $EE
    $define cCmd_Reset $E2
    $define cCmd_ScanTopBottom $C0
    $define cCmd_ScanBottomTop $C8
    $define cCmd_PowerControl $28
    $define cCmd_V5Voltage $20
    $define cCmd_Contrast $81
    $define cCmd_StaticOn $AD
    $define cCmd_StaticOff $AC

$ifndef True
    $define True 1
$endif
$ifndef False
    $define False 0
$endif
'----------------------------------------------------------------------------------------------------

   GoTo _KS0713_Main                            ' Jump over the subroutines

'****************************************************************************
'* Name    : hStrobeEN                                                      *
'* Purpose : Helper routine to Strobe the LCD's EN pin                      *
'****************************************************************************
$define hStrobeEN() GoSub _hStrobeEN
_hStrobeEN:
    Clear Glcd_CS1
    Set GLCD_EN_RD
    DelayCS 1
    Clear GLCD_EN_RD
    Set Glcd_CS1
    Return

'****************************************************************************
'* Name    : hDataMode                                                      *
'* Purpose : Helper routine to Switch to GLCD Data mode                     *
'*           Modified For KS0713                                            *
'****************************************************************************
$define hDataMode() Set GLCD_A0

'****************************************************************************
'* Name    : hInstMode                                                      *
'* Purpose : Helper routine to Switch to GLCD instruction mode              *
'*           Modified For KS0713                                            *
'****************************************************************************
$define hInstMode() Clear GLCD_A0

'****************************************************************************
'* Name    : hReadMode                                                      *
'* Purpose : Helper routine to Set LCD to Read mode                         *
'****************************************************************************
$define hReadMode() Set GLCD_RW

'****************************************************************************
'* Name    : hWriteMode                                                     *
'* Purpose : Helper routine to Set LCD to write mode                        *
'****************************************************************************
$define hWriteMode() Clear GLCD_RW

'****************************************************************************
'* Name    : hWaitForIdle                                                   *
'* Purpose : Helper routine to Block LCD access until signalled ready.      *
'* Note    : Doesn't alter WREG                                             *
'****************************************************************************
$define hWaitForIdle() GoSub _hWaitForIdle

_hWaitForIdle:
    bGlcd_WaitTimeout = $FF
    Input GLCD_Data         ' Set Data bus to Input
    hInstMode()             ' Instruction Data
    hReadMode()             ' Read mode
    Repeat
        hStrobeEN()
        If GLCD_Data.7 = 0 Then Return
        Dec bGlcd_WaitTimeout
    Until STATUS.0 = 1      ' A loop of 256
    Return

'****************************************************************************
'* Name    : hSetData                                                       *
'* Purpose : Helper routine To Write a Data Byte to the LCD                 *
'****************************************************************************
$define hSetData(pDat) bGlcd_Data = pDat : GoSub _hSetData

_hSetData:
    hWaitForIdle()          ' Block until not busy
    Output GLCD_Data        ' Set Data bus To Output
    hDataMode()             ' Access display RAM Data
    hWriteMode()            ' Write mode
    GLCD_Data = bGlcd_Data  ' Write to the bus
    hStrobeEN()             ' Write to the LCD
    Return

'****************************************************************************
'* Name    : hGetData                                                       *
'* Purpose : Helper routine to Read a Byte from the LCD                     *
'****************************************************************************
$define hGetData() _hGetData

_hGetData Macro \Byte
    GoSub __hGetData
    Return_Wreg
Endm
__hGetData:
    hWaitForIdle()          ' Block until not busy
    Input GLCD_Data         ' Set Data bus To Input
    hDataMode()             ' Access display RAM Data
    hReadMode()             ' Read mode
    hStrobeEN()             ' Latch Data
    WREG = GLCD_Data        ' Place data read into WREG ready to be returned
    Return

'****************************************************************************
'* Name    : hCommand                                                       *
'* Purpose : Helper routine to Write a command Byte to the LCD              *
'****************************************************************************
$define hCommand(pCom) bhCommand = pCom : GoSub _Command
_Command:
    hWaitForIdle()          ' Block until not busy
    Output GLCD_Data        ' Set Data bus To Output
    hInstMode()             ' Instruction mode
    hWriteMode()            ' Write mode
    GLCD_Data = bhCommand   ' Write to the bus
    hStrobeEN()             ' Write to the LCD
    Return

'****************************************************************************
'* Name    : hCommandWithData                                               *
'* Purpose : Helper routine to Write a command plus and Data Byte to LCD    *
'****************************************************************************
$define hCommandWithData(pCom, pDat) '
    bhCommand = pCom '
    bGlcd_Data = pDat '
    GoSub _hCommandWithData

_hCommandWithData:
    hWaitForIdle()          ' Block until not busy
    Output GLCD_Data        ' Set Data bus to Output
    hInstMode()             ' Instruction mode
    hWriteMode()            ' Write mode
    GLCD_Data = bhCommand   ' Write to the bus
    hStrobeEN()             ' Write to the LCD
    GLCD_Data = bGlcd_Data
    hStrobeEN()
    Return
'****************************************************************************
'* Name    : Glcd_Contrast                                                  *
'* Purpose : Adjust the contrast of the LCD                                 *
'* Input   : pConVal = Contrast value (0 to 63)                             *
'****************************************************************************

$define Glcd_Contrast(pConVal) hCommandWithData(cCmd_Contrast,pConVal)

'****************************************************************************
'* Name    : hSetPosition                                                   *
'* Purpose : Helper routine to Set LCD x and y positions                    *
'* Input   : bGlcd_Ypos = Y position (Line)                                 *
'*         : bGlcd_Xpos = X position                                        *
'* Notes   : A strong use of WREG is implemented for fast expressions       *
'*         : This is only possible by examining the ASM produced in order   *
'*         : to make sure no clashes with WREG are present                  *
'****************************************************************************
$define hSetPosition() _hSetPosition

_hSetPosition Macro
    GoSub __hSetPosition
Endm
#ifMacro _hSetPosition, _hReadByte, _Glcd_ReadByte, _Glcd_Cls, _Glcd_Plot, _Glcd_Pixel, _PrintChar
__hSetPosition:
    hCommand(bGlcd_Ypos | cCmd_Page)    ' Set Y position
$ifdef GLCD_Flip
    WREG = bGlcd_Xpos + 4
    WREG = WREG >> 4
    hCommand(WREG | cCmd_ColumnMS)      ' Set X, upper nibble
    WREG = bGlcd_Xpos + 4
    hCommand(WREG & cCmd_ColumnLS)      ' Set X, lower nibble
$else
    WREG = bGlcd_Xpos >> 4
    hCommand(WREG | cCmd_ColumnMS)      ' Set X, upper nibble
    hCommand(bGlcd_Xpos & cCmd_ColumnLS)' Set X, lower nibble
$endif
    Return
#endIfMacro
'****************************************************************************
'* Name    : hWriteByte                                                     *
'* Purpose : Helper routine to Write a Byte At x, y                         *
'* Input   : bGlcd_Ypos = Y position (Glcd_Line)                            *
'*         : bGlcd_Xpos = X position                                        *
'****************************************************************************
$define hWriteByte(pVal) hSetPosition() : hSetData(pVal)

'****************************************************************************
'* Name    : hReadByte                                                      *
'* Purpose : Helper routine to read a byte at x, y                          *
'* Input   : bGlcd_Ypos = Y position (Glcd_Line)                            *
'*         : bGlcd_Xpos = X position                                        *
'****************************************************************************
$define hReadByte() _hReadByte

_hReadByte Macro \Byte
    GoSub __hSetPosition
    GoSub __hGetData
    Return_Wreg
Endm
'-------------------------------------------------------------------------------------------------
' Write a byte to the LCD at location x,y
' Input     : bGlcd_Ypos holds the Y position (line 0 to 7)
'           : bGlcd_Xpos holds the X position
'           : WREG holds the byte to write
' Output    : None
' Notes     : None
'
$define Glcd_WriteByte(pXpos, pYpos, pByte) '
    bGlcd_Xpos = pXpos    '
    bGlcd_Ypos = pYpos    '
    hWriteByte(pByte)

'-------------------------------------------------------------------------------------------------
' Read a byte from the LCD at position x, y(line)
' Input     : bGlcd_Ypos holds the Y position (Line 0 To 7)
'           : bGlcd_Xpos holds the X position
' Output    : WREG holds the byte read from the LCD
' Notes     : None
'
$define Glcd_ReadByte(pXpos, pYpos) _Glcd_ReadByte pXpos,pYpos

_Glcd_ReadByte Macro pXpos, pYpos \Byte
    #if (Prm_1 == Byte) || (Prm_1 == Word) || (Prm_1 == Dword)
        Byte_Byte pXpos,bGlcd_Xpos
    #else
        Num_Byte pXpos,bGlcd_Xpos
    #endif

    #if (Prm_2 == Byte) || (Prm_2 == Word) || (Prm_2 == Dword)
        Byte_Wreg pYpos
    #else
        Num_Wreg pYpos
    #endif
    GoSub __Glcd_ReadByte
    Return_Wreg
Endm

#ifMacro- _Glcd_ReadByte        ' Add this block only if the macro has been used in the program
__Glcd_ReadByte:
    bGlcd_Ypos = WREG
    hSetPosition()
    GoSub __hGetData             ' Dummy call
    GoSub __hGetData
    Return
#endIfMacro-
'-------------------------------------------------------------------------------------------------
' Clear the LCD's screen
' Input     : None
' Output    : None
' Notes     : None
'
$define Glcd_Cls() _Glcd_Cls

_Glcd_Cls Macro
    GoSub __Glcd_Cls
Endm

#ifMacro- _Glcd_Cls             ' Add this block only if the macro has been used in the program
__Glcd_Cls:
   bGlcd_Ypos = 0
$ifdef GLCD_Flip                ' Use flipped display?
    bGlcd_Xpos = 4              ' Yes.
$else
    bGlcd_Xpos = 0              ' No.
$endif
   Repeat
	  WREG = bGlcd_Ypos | cCmd_Page
      hCommand(WREG)
      bGlcd_Xpos = 0
      Repeat
         '
         ' This is a special case where it is known that WREG is safe to use in this manner
         '
         WREG = bGlcd_Xpos >> 4
         WREG = WREG | cCmd_ColumnMS
         hCommand(WREG)
         WREG = bGlcd_Xpos & cCmd_ColumnLS
         hCommand(WREG)
         hSetData($00)
         Inc bGlcd_Xpos
      Until bGlcd_Xpos >= 130
	  Inc bGlcd_Ypos
   Until bGlcd_Ypos > 7
   bGlcd_Xpos = 0
   bGlcd_Ypos = 0
   hSetPosition()
   Return
#endIfMacro-
'-------------------------------------------------------------------------------------------------
' Set Pixel At Pixel location x,y
' Input     : bGlcd_Ypos holds the Y position
'           : bGlcd_Xpos holds the X position
'           : WREG holds the colour of the Pixel
' Output    : None
' Notes     : None
'
$define Glcd_Plot(pXpos, pYpos, pColour) '
    bGlcd_PixelX = pXpos '
    bGlcd_PixelY = pYpos '
    WREG = pColour '
    _Glcd_Plot

_Glcd_Plot Macro
    GoSub __Glcd_Plot
Endm
#ifMacro- _Glcd_Plot, _Glcd_Line, _Glcd_LineTo, _Glcd_Circle, _Glcd_Box, _Glcd_Rectangle, _Glcd_Vline, _Glcd_Hline
__Glcd_Plot:
    bColour = WREG                              ' Place WREG into its working variable
    If bGlcd_PixelX < cGlcd_Width Then          ' \ Test boundaries
        If bGlcd_PixelY < cGlcd_Height Then     ' /
            bGlcd_Ypos = bGlcd_PixelY / 8
            bGlcd_Xpos = bGlcd_PixelX
            hSetPosition()
            hGetData()
            bGlcd_gPixel = hGetData()
            PRODH = bGlcd_PixelY // 8
            PRODH = 1 << PRODH
            If bColour = 0 Then                 ' Pixel is white?
                WREG = ~PRODH
                bGlcd_gPixel = bGlcd_gPixel & WREG
            ElseIf bColour = 1 Then             ' Pixel is black?
                bGlcd_gPixel = bGlcd_gPixel | PRODH
            Else                                ' Pixel is Xor?
                bGlcd_gPixel = bGlcd_gPixel ^ PRODH
            EndIf
            hSetPosition()
            hSetData(bGlcd_gPixel)
        EndIf
    EndIf
    Return
#endIfMacro-
'-------------------------------------------------------------------------------------------------
' Return the pixel colour at pixel position x, y
' Input     : bGlcd_Ypos holds the Y position
'           : bGlcd_Xpos holds the X position
' Output    : WREG holds the colour of the pixel
' Notes     : None
'
Byte_Math = On  ' Always create byte sized expressions
$define Glcd_Pixel(pXpos, pYpos) _Glcd_Pixel pXpos,pYpos

_Glcd_Pixel Macro pXpos, pYpos \Byte
    #if (Prm_1 == Byte) || (Prm_1 == Word) || (Prm_1 == Dword)
        Byte_Byte pXpos,bGlcd_Xpos
    #else
        Num_Byte pXpos,bGlcd_Xpos
    #endif

    #if (Prm_2 == Byte) || (Prm_2 == Word) || (Prm_2 == Dword)
        Byte_Wreg pYpos
    #else
        Num_Wreg pYpos
    #endif
    GoSub __Glcd_Pixel
    Return_Wreg
Endm
#ifMacro- _Glcd_Pixel                   ' Add this block only if the macro has been used in the program
__Glcd_Pixel:
    PRODH = WREG                        ' Save WREG (y parameter)
    bGlcd_Ypos = WREG / 8
    hSetPosition()
    hGetData()
    bGlcd_gPixel = hGetData()
    WREG = bGlcd_gPixel >> (PRODH // 8)
    Return
#endIfMacro-
Byte_Math = Off ' Back to safe expressions
'-------------------------------------------------------------------------------------------------
' Move to character on the GLCD
' Input     : bGlcd_Ypos = Y position (Line 0 to 7)
'           : bGlcd_Xpos = X position (0 to 21)
' Output    : None
' Notes     : X and Y positions depend on the font used
'
$define Glcd_Cursor(pXpos, pYpos) '
    bGlcd_Ypos = pYpos '
    WREG = pXpos '
    _Glcd_Cursor

_Glcd_Cursor Macro
    GoSub __Glcd_Cursor
Endm

#ifMacro- _Glcd_Cursor                  ' Add this block only if the macro has been used in the program
__Glcd_Cursor:
    bGlcd_Xpos = WREG * 6
    Return
#endIfMacro-
'-------------------------------------------------------------------------------------------------
' Display an ASCII character
' Input     : WREG holds the character to display
'           : bGlcd_Ypos holds the Y position (Line 0 to 7)
'           : bGlcd_Xpos holds the X position
' Output    : WREG still contains the character that was displayed
' Notes     : None
'
$define Glcd_PrintChar(pChar) WREG = pChar : _PrintChar

_PrintChar Macro
    GoSub __Glcd_PrintChar
Endm

#ifdef Print#Req
@Print
__Glcd_PrintChar:
    bGlcd_CharSave = WREG               ' Save WREG for later
    wGlcd_TblPtrSave = wGlcd_Tblptr     ' Save the contents of the register pair TBLPTRL\H
'
' Character's address = bGlcd_CharSave * Font size
' bGlcd_CharSave holds the LCD byte to display
' Add the offset for the address of Font: wGlcd_CharOffset + Font label's address
' This is the code that requires altering for different sized fonts
'
    If bGlcd_Xpos >= 126 Then           ' Has the Xpos gone off screen?
        Inc bGlcd_Ypos                  ' Yes. So increment to the next line
        bGlcd_Xpos = 0                  ' Reset the X position
        If bGlcd_Ypos > 7 Then          ' Has the Ypos (line) gone off screen?
            bGlcd_Ypos = 0              ' Yes. So reset the Y position (line)
        EndIf
    EndIf
    bGlcd_LcdLoop = 0                   ' Point to the first byte of the character's font
    Repeat                              ' Form a loop for the amount of bytes in the charcater
        wGlcd_CharOffset = bGlcd_CharSave * 6           ' Font is 6 pixels wide
        wGlcd_Tblptr = wGlcd_CharOffset + bGlcd_LcdLoop ' Find the location of the charcater byte in the font table
        TABLAT = LRead8 Font[wGlcd_Tblptr]              ' Extract the byte from the font table
        hWriteByte(TABLAT)               ' Send the value in TABLAT to the LCD
        Inc bGlcd_Xpos                  ' Increment the X position
        Inc bGlcd_LcdLoop               ' Next byte
    Until bGlcd_LcdLoop >= 6            ' Have we done 6 writes yet ?
    wGlcd_Tblptr = wGlcd_TblPtrSave     ' Restore the contents of the register pair TBLPTRL\H
    WREG = bGlcd_CharSave               ' Restore the value of WREG
    Return
#endif
'-------------------------------------------------------------------------------------------------
' Display text
' Input     : Uses a single modifier from the compiler's Print routine
' Output    : None
' Notes     : This routine hi-jacks the compiler's Print library
'
$define Glcd_Print(pStringName) _Glcd_Print : Print pStringName
'
' Create a dummy macro to trigger the inclusion of the font table
'
_Glcd_Print Macro pStringName
Endm

'-------------------------------------------------------------------------------------------------
' Display text at a given X and Y location
'
' Input     : bGlcd_Ypos holds the Y position (Line 0 To 7)
'           : bGlcd_Xpos holds the X position
'           : Uses a single modifier from the compiler's Print routine
' Output    : None
' Notes     : This routine hi-jacks the compiler's Print library
'
$define Glcd_PrintAt(pXpos, pYpos, pStringName) '
    Glcd_Cursor(pXpos, pYpos) '
    Glcd_Print(pStringName)

'-------------------------------------------------------------------------------------------------
' Glcd_Circle
' Draw a Glcd_Circle on the Graphic LCD
' Input     : bGlcd_Xpos_S holds the starting X position
'           : bGlcd_Ypos_S holds the starting Y position
'           : WREG holds the radius of the circle
'           : bColour holds the colour of the pixel(0 = off, 1 = on, 2 = xor)
' Output    : None
' Notes     : None
'
$define Glcd_Circle(pXpos, pYpos, pRadius, pColor) '
    bGlcd_Xpos_S = pXpos '
    bGlcd_Ypos_S = pYpos '
    bColour = pColor '
    WREG = pRadius '
    _Glcd_Circle

_Glcd_Circle Macro
    GoSub __Glcd_Circle
Endm

#ifMacro- _Glcd_Circle                              ' Add this block only if the macro has been used in the program
__Glcd_Circle:
    bGlcd_Size = WREG                               ' Transfer WREG into its working variable
    If bGlcd_Size = 0 Then Return                   ' Trap a radius of 0
    bGlcd_DD = bGlcd_Ypos_S - bGlcd_Xpos_S
    bGlcd_XX = 0
    bGlcd_YY = bGlcd_Size
    bGlcd_TR = 3 - (2 * bGlcd_Size)
    While bGlcd_XX <= bGlcd_YY
        bGlcd_QuadA = bGlcd_Xpos_S + bGlcd_XX
        bGlcd_QuadB = bGlcd_Ypos_S + bGlcd_YY
        bGlcd_QuadC = bGlcd_Xpos_S - bGlcd_XX
        bGlcd_QuadD = bGlcd_Ypos_S - bGlcd_YY
        GoSub PlotIt
        bGlcd_QuadA = bGlcd_Ypos_S + bGlcd_YY - bGlcd_DD
        bGlcd_QuadB = bGlcd_Ypos_S + bGlcd_XX
        bGlcd_QuadC = bGlcd_Ypos_S - bGlcd_YY - bGlcd_DD
        bGlcd_QuadD = bGlcd_Ypos_S - bGlcd_XX
        GoSub PlotIt
        If bGlcd_TR.7 = 1 Then
            bGlcd_TR = bGlcd_TR + 6
            bGlcd_TR = bGlcd_TR + (4 * bGlcd_XX)
        Else
            bGlcd_TR = bGlcd_TR + 10
            bGlcd_TR = bGlcd_TR + (4 * (bGlcd_XX - bGlcd_YY))
            Dec bGlcd_YY
        EndIf
        Inc bGlcd_XX
    Wend
    Return
'
' Plot 4 locations simultaneously
' This is part of the Glcd_Circle subroutine
'
PlotIt:
    Glcd_Plot(bGlcd_QuadA, bGlcd_QuadB, bColour)
    Glcd_Plot(bGlcd_QuadA, bGlcd_QuadD, bColour)
    Glcd_Plot(bGlcd_QuadC, bGlcd_QuadB, bColour)
    Glcd_Plot(bGlcd_QuadC, bGlcd_QuadD, bColour)
    Return
#endIfMacro-
'-------------------------------------------------------------------------------------
' Draw or Erase a line from bGlcd_Xpos_S, bGlcd_Ypos_S to bGlcd_Xpos_E, bGlcd_Ypos_E
'
' Input     : bGlcd_Xpos_S = Starting X position of the Line
'           : bGlcd_Ypos_S = Starting Y position of the Line
'           : bGlcd_Xpos_E = Ending X position of the Line
'           : WREG = Ending Y position of the Line
'           : bColour = Colour of the pixel(0 = off, 1 = on, 2 = xor)
' Output    : None
' Notes     :
'
$define Glcd_Line(pStartXpos, pStartYpos, pEndXpos, pEndYpos, pColor) '
    bGlcd_Xpos_S = pStartXpos '
    bGlcd_Ypos_S = pStartYpos '
    bGlcd_Xpos_E = pEndXpos '
    bColour = pColor '
    WREG = pEndYpos '
    _Glcd_Line

_Glcd_Line Macro
    GoSub __Glcd_Line
Endm

#ifMacro- _Glcd_Line, _Glcd_LineTo, _Glcd_Box, _Glcd_Rectangle ' Add this block only if the macros have been used in the program
__Glcd_Line:
    bGlcd_Ypos_E = WREG                                 ' Transfer WREG into it's working variable
    bGlcd_DeltaY = bGlcd_Ypos_E - bGlcd_Ypos_S          ' Find the initial Y direction
    bGlcd_DeltaX = bGlcd_Xpos_E - bGlcd_Xpos_S          ' Find the initial X direction

    bGlcd_StepY = 1                                     ' Default to a positive step for Ypos
    If bGlcd_DeltaY.7 = 1 Then                          ' Check if negative
        bGlcd_DeltaY = -bGlcd_DeltaY                    ' Negate bGlcd_DeltaY
        bGlcd_StepY = -1                                ' And indicate a negative step
    EndIf
    bGlcd_StepX = 1                                     ' Default to a positive step for Xpos
    If bGlcd_DeltaX.7 = 1 Then                          ' Check if negative
        bGlcd_DeltaX = -bGlcd_DeltaX                    ' Negate bGlcd_DeltaX
        bGlcd_StepX = -1                                ' And indicate a negative step
    EndIf
    Glcd_Plot(bGlcd_Xpos_S,bGlcd_Ypos_S,bColour)        ' The first pixel plot is a special case
    If bGlcd_DeltaX > bGlcd_DeltaY Then
        bGlcd_Deviation = bGlcd_DeltaY - bGlcd_DeltaX
        While bGlcd_Xpos_S <> bGlcd_Xpos_E
            If bGlcd_Deviation.7 = 0 Then               ' Check if negative
                bGlcd_Ypos_S = bGlcd_Ypos_S + bGlcd_StepY           ' Move on the Y axis directed by bGlcd_StepY
                bGlcd_Deviation = bGlcd_Deviation - bGlcd_DeltaX
            EndIf
            bGlcd_Xpos_S = bGlcd_Xpos_S + bGlcd_StepX               ' Move on the X axis directed by bGlcd_StepX
            bGlcd_Deviation = bGlcd_Deviation + bGlcd_DeltaY
            Glcd_Plot(bGlcd_Xpos_S,bGlcd_Ypos_S,bColour)            ' Draw or Erase the pixel at this position
        Wend
        Return                                          ' Return prematurely (this saves an else command)
    EndIf
    bGlcd_Deviation = bGlcd_DeltaX - bGlcd_DeltaY

    While bGlcd_Ypos_S <> bGlcd_Ypos_E
        If bGlcd_Deviation.7 = 0 Then                    ' Check if negative
            bGlcd_Xpos_S = bGlcd_Xpos_S + bGlcd_StepX              ' Move on the X axis directed by bGlcd_StepX
            bGlcd_Deviation = bGlcd_Deviation - bGlcd_DeltaY
        EndIf
        bGlcd_Ypos_S = bGlcd_Ypos_S + bGlcd_StepY                  ' Move on the Y axis directed by bGlcd_StepY
        bGlcd_Deviation = bGlcd_Deviation + bGlcd_DeltaX
        Glcd_Plot(bGlcd_Xpos_S,bGlcd_Ypos_S,bColour)      ' Draw or Erase the pixel at this position
    Wend
    Return
#endIfMacro-
'-----------------------------------------------------------------------------------------------
' Draw a line from the previous end X and Y positions to new X and Y coordinates
'
' Input     : PRODL holds the X position start
'           : WREG holds the Y position start
'           : bColour = Colour of the Pixel(0 = Off, 1 = On, 2 = xor)
' Output    : None
' Notes     : None
'
$define Glcd_LineTo(pEndXpos,pEndYpos,pColor) '
    PRODL = pEndXpos '
    bColour = pColor '
    WREG = pEndYpos '
    _Glcd_LineTo

_Glcd_LineTo Macro
    GoSub __Glcd_LineTo
Endm

#ifMacro- _Glcd_LineTo          ' Add this block only if the macro has been used in the program
__Glcd_LineTo:
    bGlcd_Ypos_E = WREG         ' Load parameter end Y pos
    bGlcd_Xpos_S = bGlcd_Xpos_E ' Transfer previous X pos end to X pos start
    bGlcd_Xpos_E = PRODL        ' Load parameter X pos end
    WREG = bGlcd_Ypos_E         ' Transfer the previous Y pos to the Y pos start
    GoSub __Glcd_Line           ' Draw a line
    Return
#endIfMacro-
'-----------------------------------------------------------------------------------------------
' Draw a vertical line
'
' Input     : bGlcd_PixelX = Starting X position of the Line
'           : bGlcd_PixelY = Starting Y position of the Line
'           : bGlcd_Size holds the length of the line
'           : bColour = Colour of the Pixel(0 = Off, 1 = On, 2 = xor)
' Output    : None
' Notes     : None
'
$define Glcd_Vline(pXpos, pYpos, pSize, pColor) '
    bGlcd_PixelX = pXpos '
    bGlcd_PixelY = pYpos '
    bColour = pColor '
    WREG = pSize '
    _Glcd_Vline

_Glcd_Vline Macro
    GoSub __Glcd_Vline
Endm

#ifMacro- _Glcd_Vline                       ' Add this block only if the macro has been used in the program
__Glcd_Vline:
    bGlcd_Size = WREG
    Repeat
        Glcd_Plot(bGlcd_PixelX, bGlcd_PixelY, bColour)
        Inc bGlcd_PixelY
        Dec bGlcd_Size
    Until bGlcd_Size = 255
    Return
#endIfMacro-
'-----------------------------------------------------------------------------------------------
' Draw a horizontal line
'
' Input     : bGlcd_PixelX = Starting X position of the Line
'           : bGlcd_PixelY = Starting Y position of the Line
'           : bGlcd_Size holds the length of the line
'           : bColour = Colour of the Pixel(0 = Off, 1 = On, 2 = xor)
' Output    : None
' Notes     : None
'
$define Glcd_Hline(pXpos, pYpos, pSize, pColor) '
    bGlcd_PixelX = pXpos '
    bGlcd_PixelY = pYpos '
    bColour = pColor '
    WREG = pSize '
    _Glcd_Hline

_Glcd_Hline Macro
    GoSub __Glcd_Hline
Endm

#ifMacro- _Glcd_Hline                       ' Add this block only if the macro has been used in the program
__Glcd_Hline:
    bGlcd_Size = WREG
    Repeat
        Glcd_Plot(bGlcd_PixelX, bGlcd_PixelY, bColour)
        Inc bGlcd_PixelX
        Dec bGlcd_Size
    Until bGlcd_Size = 255
    Return
#endIfMacro-
'-----------------------------------------------------------------------------------------------
' Draw a square
' Input     : bGlcd_Xpos_S holds the starting X position
'           : bGlcd_Ypos_S holds the starting Y position
'           : WREG holds the size of the box
'           : bColour holds the colour of the pixel(0 = off, 1 = on, 2 = xor)
' Output    : None
' Notes     :
'
$define Glcd_Box(pXpos, pYpos, pSize, pColor) '
    bGlcd_Xpos_S = pXpos '
    bGlcd_Ypos_S = pYpos '
    bColour = pColor '
    WREG = pSize '
    _Glcd_Box

_Glcd_Box Macro
    GoSub __Glcd_Box
Endm

#ifMacro- _Glcd_Box                         ' Add this block only if the macro has been used in the program
__Glcd_Box:
    bGlcd_Size = WREG
    Glcd_Line(bGlcd_Xpos_S, bGlcd_Ypos_S, bGlcd_Xpos_S + bGlcd_Size, bGlcd_Ypos_S, bColour)  ' Draw top line
    bGlcd_Ypos_S = bGlcd_Ypos_E             ' Transfer previous Y pos end to Y pos start
    bGlcd_Xpos_S = bGlcd_Xpos_E             ' Transfer previous X pos end to X pos start
    WREG = bGlcd_Ypos_E + bGlcd_Size
    GoSub __Glcd_Line                       ' Draw right line

    bGlcd_Ypos_S = bGlcd_Ypos_E             ' Transfer previous Y pos end to Y pos start
    bGlcd_Xpos_S = bGlcd_Xpos_E             ' Transfer previous X pos end to X pos start
    bGlcd_Xpos_E = bGlcd_Xpos_E - bGlcd_Size
    WREG = bGlcd_Ypos_E
    GoSub __Glcd_Line                       ' Draw bottom line

    bGlcd_Ypos_S = bGlcd_Ypos_E             ' Transfer previous Y pos end to Y pos start
    bGlcd_Xpos_S = bGlcd_Xpos_E             ' Transfer previous X pos end to X pos start
    WREG = bGlcd_Ypos_E - bGlcd_Size
    GoSub __Glcd_Line                       ' Draw left line
    Return
#endIfMacro-
'-----------------------------------------------------------------------------------------------
' Draw a Rectangle
' Input     : bGlcd_Xpos_S holds the starting X position
'           : bGlcd_Ypos_S holds the starting Y position
'           : bGlcd_Length holds the length of the rectangle
'           : WREG holds the height of the rectangle
'           : bColour holds the colour of the pixel(0 = off, 1 = on, 2 = xor)
' Output    : None
' Notes     :
'
$define Glcd_Rectangle(pXpos, pYpos, pLength, pHeight, pColor) '
    bGlcd_Xpos_S = pXpos '
    bGlcd_Ypos_S = pYpos '
    bGlcd_Length = pLength '
    bColour = pColor '
    WREG = pHeight '
    _Glcd_Rectangle

_Glcd_Rectangle Macro
    GoSub __Glcd_Rectangle
Endm

#ifMacro- _Glcd_Rectangle                   ' Add this block only if the macro has been used in the program
__Glcd_Rectangle:
    bGlcd_Height = WREG
    Glcd_Line(bGlcd_Xpos_S, bGlcd_Ypos_S, bGlcd_Xpos_S + bGlcd_Length, bGlcd_Ypos_S, bColour)  ' Draw top line

    bGlcd_Ypos_S = bGlcd_Ypos_E             ' Transfer previous Y pos end to Y pos start
    bGlcd_Xpos_S = bGlcd_Xpos_E             ' Transfer previous X pos end to X pos start
    WREG = bGlcd_Ypos_E + bGlcd_Height
    GoSub __Glcd_Line                       ' Draw right line

    bGlcd_Ypos_S = bGlcd_Ypos_E             ' Transfer previous Y pos end to Y pos start
    bGlcd_Xpos_S = bGlcd_Xpos_E             ' Transfer previous X pos end to X pos start
    bGlcd_Xpos_E = bGlcd_Xpos_E - bGlcd_Length
    WREG = bGlcd_Ypos_E
    GoSub __Glcd_Line                       ' Draw bottom line

    bGlcd_Ypos_S = bGlcd_Ypos_E             ' Transfer previous Y pos end to Y pos start
    bGlcd_Xpos_S = bGlcd_Xpos_E             ' Transfer previous X pos end to X pos start
    WREG = bGlcd_Ypos_E - bGlcd_Height
    GoSub __Glcd_Line                       ' Draw left line
    Return
#endIfMacro-
'-----------------------------------------------------------------------------------------------
' Initialise the KS0713 graphic LCD
' Input     : None
' Output    : None
' Notes     : None
'
$define Glcd_Init() _Glcd_Init
_Glcd_Init Macro
    GoSub __Glcd_Init
Endm

#ifMacro- _Glcd_Init                    ' Add this block only if the macro has been used in the program
__Glcd_Init:
    bGlcd_Xpos = 0
    bGlcd_Ypos = 0
    DelayMS cGlcd_InitDelay             ' Start up delay, allow GLCD to settle
    Output GLCD_RW                      ' Read/Write is output
    Low GLCD_EN_RD                      ' Enable is low
    Output GLCD_A0                      ' Data or instruction is low (command mode)
    Output Glcd_CS1                     ' Screen
    High GLCD_Reset                     ' Reset

    hCommand(cCmd_Reset)                ' Software reset
    hCommand(cCmd_Bias19)
$ifdef GLCD_Flip
    hCommand(cCmd_ADCReverse)           ' ADC Output = reversed
$else
    hCommand(cCmd_ADCNormal)            ' ADC Output = normal
$endif
    hCommand(cCmd_DisplayAllOff)        ' Display all point normal
    hCommand(cCmd_V5Voltage | 5)        ' Internal resistor ratio (0..7)
    hCommand(cCmd_PowerControl | $07)   ' Internal power supply mode
    hCommandWithData(cCmd_Contrast,26)  ' LCD contrast
    hCommand(cCmd_StaticOff)            ' Static indicator is off
    hCommand(cCmd_On)                   ' Display is on
$ifdef GLCD_Flip
    hCommand(cCmd_ScanBottomTop)        ' Scan bottom to top
$else
    hCommand(cCmd_ScanTopBottom)        ' Scan top to bottom
$endif

$ifdef GLCD_Negative
    hCommand(cCmd_DisplayReverse)       ' Display negative polarity
$else
    hCommand(cCmd_DisplayNormal)        ' Display normal polarity
$endif
    hCommand(cCmd_StartLine)            ' Display start line (Or value for line number)
    Return
#endIfMacro
'-------------------------------------------------------------------------------------------------
' Load the font table into the program

#ifMacro- _PrintChar, _Glcd_Print, _Glcd_Cursor
    Include "Font.inc"
#endIfMacro-
'-----------------------------------------------------------------------------------------------
_KS0713_Main:
    Glcd_Init()

Warnings = On
Reminders = On

$endif

