'
' Graphic LCD demo using the KS0713 chipset and the KS0713 library
'
' Written by Les Johnson - 31st May 2010
' For use with the Crownhill Amicus18 BASIC compiler version 1.0.0.8 onwards
'
' This source code is supplied as-is, and no guarantees are offered on it's operation
'
    Optimiser_Level = 3                     ' Maximum optimisation
    Float_Display_Type = Fast               ' Fast and more accurate floating point display required
'
' User defines must be placed before the KS0713 include directive
' 
    $define GLCD_CS1    PORTC.2             ' LCD's CS1 pin
    $define GLCD_Reset  PORTC.4             ' LCD's Reset pin
    $define GLCD_A0     PORTC.1             ' LCD's A0 pin
    $define GLCD_RW     PORTA.5 'PORTC.3    ' LCD's RW
    $define GLCD_EN_RD  PORTA.4 'PORTC.0    ' LCD's En or RD pin
    $define GLCD_Data   PORTB               ' LCD's data port

    Include "KS0713.inc"                    ' Load the KS0713 graphic LCD routines into the program
'
' Demo Variables
'    
    Dim tTemp As Bit   
    Dim bXpos As Byte                       ' Holds the X position of lines and pixels etc...
    Dim bYpos As Byte                       ' Holds the Y position of lines and pixels etc...
    Dim bXposEnd As Byte
    Dim bYposEnd As Byte
    Dim bRadius As Byte                     ' Holds the radius of a circle
    Dim bSize As bRadius                    ' Holds the size of a a square   
    Dim bLength As bRadius                  ' Holds the length of a rectangle
    Dim bHeight As Byte                     ' Holds the height of a rectangle
    
    Dim bTemp As Byte                       ' Create a byte sized variable
    Dim wTemp As Word                       ' Create a word sized variable
    Dim fTemp As Float                      ' Create a floating point variable
    
'-----------------------------------------------------------------------------------------------
' Start of the main demo program 
Main:
    Glcd_Cls()                      ' Clear the LCD's screen
'
' Fill the screen with lines
'
    For bYposEnd = 0 To 63 Step 2
        Glcd_Line(0,0,127,bYposEnd,1)
        DelayMS 10
    Next
'
' Clear the screen of lines
'
    For bYposEnd = 0 To 63 Step 2
        Glcd_Line(0,0,127,bYposEnd,0)
        DelayMS 10
    Next 
'
' Fill the screen with pixels
'
    For bXpos = 0 To 127 Step 3
        For bYpos = 0 To 63 Step 3
            Glcd_Plot(bXpos,bYpos,1)
            DelayMS 5
        Next
    Next
'
' Clear the screen of pixels using xor
'
    For bXpos = 0 To 127 Step 3
        For bYpos = 0 To 63 Step 3
            Glcd_Plot(bXpos,bYpos,2)
            DelayMS 5
        Next
    Next   
'
' Draw a collection of circles
'
    For bRadius = 1 To 63 Step 3
        Glcd_Circle(63,32,bRadius,1)
        DelayMS 50
    Next
'
' Erase a collection of circles
'
    For bRadius = 1 To 63 Step 3
        Glcd_Circle(63,32,bRadius,0)
        DelayMS 50
    Next
'
' Draw a collection of squares
'
    For bSize = 1 To 127 Step 3
        Glcd_Box(0,0,bSize,1)
        DelayMS 50
    Next
'
' Erase a collection of squares
'
    For bSize = 1 To 127 Step 3
        Glcd_Box(0,0,bSize,0)
        DelayMS 50
    Next    
'
' Draw a collection of rectangles
'
    For bLength = 1 To 127 Step 3
        For bHeight = 1 To 63 Step 3
            Glcd_Rectangle(0,0,bLength, bHeight,1)
        Next
    Next
'
' Copy and re-draw the current screen pixel by pixel and invert it
'
    For bYpos = 0 To 63
        For bXpos = 0 To 127
            tTemp = Glcd_Pixel(bXpos,bYpos)
            Glcd_Plot(bXpos, bYpos, tTemp + 1)
        Next
    Next
'
' Clear the screen
'    
    Glcd_Cls
'
' Display some text
'        
    bTemp = 0                           ' Reset bTemp
    wTemp = 0                           ' Reset wTemp
    fTemp = 0                           ' Reset fTemp

    While 1 = 1                         ' Create an infinite loop
        Glcd_PrintAt(0, 1, "Byte: ")    ' Display text at the start of the second line 
        Glcd_Print(Dec bTemp)           ' Convert an integer to ASCII and display it
        Glcd_Print("  ")
 
        Glcd_PrintAt(0, 2, "Word: ")    ' Display text at the start of the third line
        Glcd_Print(Dec wTemp)           ' Convert an integer to ASCII and display it
        Glcd_Print("    ")

        Glcd_PrintAt(0, 3, "Float: ")   ' Display text at the start of the fourth line
        Glcd_Print(Dec1 fTemp)          ' Convert a float to ASCII and display it

        Inc bTemp                       ' Increment bTemp
        Inc wTemp                       ' Increment wTemp
        fTemp = fTemp + 0.2             ' Increment fTemp
 
        DelayMS 100                     ' Slow things down so we can see the text change
    Wend                                ' Do it forever

